﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;
using System.Net.Http.Headers;
using Newtonsoft.Json;
namespace BulkSMSAPI
{
    class BaseResponse
    {
        public Boolean success { get; set; }
        public string message { get; set; }
    }
    class RequestData
    {
        public int request_id { get; set; }
    }
    class StatusData
    {
        public int request_id { get; set; }
        public string agent_name { get; set; }
        public string telephone { get; set; }
        public string smses { get; set; }
        public string status { get; set; }
        public string started_at { get; set; }
        public string ended_at { get; set; }
        public string acquired_at { get; set; }
        public string queued_at { get; set; }
        public string canceled_at { get; set; }
        public string updated_at { get; set; }
    }
    class RequestsData
    {
        public int request_id { get; set; }
        public string status { get; set; }
        public string telephone { get; set; }
        public string queued_at { get; set; }
        public string updated_at { get; set; }
    }
    class SMSesByTIDData
    {
        public int telephone_id { get; set; }
        public int request_id { get; set; }
        public string telephone { get; set; }
        public string smses { get; set; }
        public string updated_at { get; set; }
    }
    class TelephonesData
    {
        public int telephone_id { get; set; }
        public string telephone { get; set; }
        public string added_at { get; set; }
        public string expired_at { get; set; }
    }
    class RequestResponse : BaseResponse
    {
        public RequestData data { get; set; }
    }
    class StatusResponse : BaseResponse
    {
        public StatusData data { get; set; }
    }
    class TelephonesResponse : BaseResponse
    {
        public List<TelephonesData> data { get; set; }
    }
    class RequestsResponse : BaseResponse
    {
        public List<RequestsData> data { get; set; }
    }
    class SMSesByTIDResponse: BaseResponse
    {
        public List<SMSesByTIDData> data { get; set; }
    }
    class CancelResponse : BaseResponse
    {
        public RequestData data { get; set; }
    }
    class AutocheckResponse : BaseResponse
    {
    }

    class NCStatusData
    {
        public int request_id { get; set; }
        public string agent_name { get; set; }
        public string last_telephone { get; set; }
        public string telephone { get; set; }
        public string status { get; set; }
        public string submitted_at { get; set; }
        public string acquired_at { get; set; }
        public string processed_at { get; set; }
        public string completed_at { get; set; }
        public string canceled_at { get; set; }
        public string updated_at { get; set; }
    }
    class NCRequestsData
    {
        public int request_id { get; set; }
        public string status { get; set; }
        public string last_telephone { get; set; }
        public string telephone { get; set; }
        public string submitted_at { get; set; }
        public string canceled_at { get; set; }
        public string updated_at { get; set; }
    }
    class NCRequestResponse : BaseResponse
    {
        public RequestData data { get; set; }
    }
    class NCStatusResponse : BaseResponse
    {
        public NCStatusData data { get; set; }
    }
    class NCRequestsResponse : BaseResponse
    {
        public List<NCRequestsData> data { get; set; }
    }
    class NCCancelResponse : BaseResponse
    {
        public RequestData data { get; set; }
    }

    class Client : IDisposable
    {
        private HttpClient client;
        public string url = "";
        public Client(string base_url,string key)
        {
            client = new HttpClient();
            url = base_url + key;
            client.BaseAddress = new Uri(url);
            client.DefaultRequestHeaders.Accept.Clear();
            client.DefaultRequestHeaders.Accept.Add(
                new MediaTypeWithQualityHeaderValue("application/json"));
        }
        public void Dispose()
        {
            client.Dispose();
        }

        public async Task<RequestResponse> Request(int telephone_id)
        {
            HttpResponseMessage response = await client.GetAsync(url + "/bulk_sms/request/" + telephone_id);
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<RequestResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<StatusResponse> Status(int request_id)
        {
            HttpResponseMessage response = await client.GetAsync(url + "/bulk_sms/status/" + request_id);
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<StatusResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<TelephonesResponse> Telephones()
        {
            HttpResponseMessage response = await client.GetAsync(url + "/bulk_sms/telephones");
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<TelephonesResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<RequestsResponse> Requests()
        {
            HttpResponseMessage response = await client.GetAsync(url + "/bulk_sms/requests");
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<RequestsResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<SMSesByTIDResponse> SMSesByTID(int telephone_id)
        {
            HttpResponseMessage response = await client.GetAsync(url + "/bulk_sms/smses_by_tid/" + telephone_id);
            response.EnsureSuccessStatusCode();
            Console.WriteLine(await response.Content.ReadAsStringAsync());
            return JsonConvert.DeserializeObject<SMSesByTIDResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<CancelResponse> Cancel(int request_id)
        {
            HttpResponseMessage response = await client.GetAsync(url + "/bulk_sms/cancel/" + request_id);
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<CancelResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<AutocheckResponse> EnableAutocheck(int telephone_id)
        {
            HttpResponseMessage response = await client.GetAsync(url + "/bulk_sms/autocheck/" + telephone_id + "/enable");
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<AutocheckResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<AutocheckResponse> DisableAutocheck(int telephone_id)
        {
            HttpResponseMessage response = await client.GetAsync(url + "/bulk_sms/autocheck/" + telephone_id + "/disable");
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<AutocheckResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<NCRequestResponse> NCRequest(int telephone_id, string zip)
        {
            HttpResponseMessage response = await client.GetAsync(url + "/nc/request/" + telephone_id + "/" + zip);
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<NCRequestResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<NCStatusResponse> NCStatus(int request_id)
        {
            HttpResponseMessage response = await client.GetAsync(url + "/nc/status/" + request_id);
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<NCStatusResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<NCRequestsResponse> NCRequests()
        {
            HttpResponseMessage response = await client.GetAsync(url + "/nc/requests");
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<NCRequestsResponse>(await response.Content.ReadAsStringAsync());
        }
        public async Task<NCCancelResponse> NCCancel(int request_id)
        {
            HttpResponseMessage response = await client.GetAsync(url + "/nc/cancel/" + request_id);
            response.EnsureSuccessStatusCode();
            return JsonConvert.DeserializeObject<NCCancelResponse>(await response.Content.ReadAsStringAsync());
        }
    }
}